/*
 *  PlantAttack.h
 *  GOSM
 *
 *  Created by Ivica Aracic on 01.06.10.
 *  Copyright 2010 Ivica Aracic. All rights reserved.
 *
 */

#ifndef PlantAttack_H_
#define PlantAttack_H_

#include "gosm.h"

#include <iostream>

using namespace gosm;

struct AttackSettings {
	float PlantAttackInactiveTimer;
	float PlantAttackAppearTimer;
	float PlantAttackEffectiveTimer;
	float PlantAttackGrowTimer;
	float PlantAttackBiteTimer;
	float PlantAttackBiteCooldownTimer;
	float PlantAttackDisappearTimer;
};


/////////////////////////////////////

class PlantAttack {
public:
	typedef PlantAttack THIS;
	
	typedef State<THIS> State;
	typedef StateMemberFnPtr<THIS> StateFnPtr;
	
public:
	enum STATE_ID {
		STATE_ID_Inactive,
		STATE_ID_Appear,
		STATE_ID_Effective,
		STATE_ID_Grow,
		STATE_ID_Bite,
		STATE_ID_BiteCooldown,
		STATE_ID_Disappear
	};

public:
	// TODO: 
	// 1/ check how this behaves when used in an inheritance hierarchy of GOs
	// add possibility to remove/add rules
	// 2/ more than one sm in a GO?
	StateMachineDef<THIS>* getStateMachineDef() {		
		if(smDef) return smDef;
		
		static AttackSettings s = {0, 1, 1, 1, 1, 1, 1};
		static StateMachineDef<THIS>* smDef = StateMachineDef<THIS>::begin();
		
		State* root = smDef->getRoot();
		
		static State      Init		(GOSM_ID(Init),		root);
		static StateFnPtr Inactive	(GOSM_ID(Inactive),	root, &s.PlantAttackInactiveTimer,		&THIS::stateInactive);
		static StateFnPtr Appear	(GOSM_ID(Appear),	root, &s.PlantAttackAppearTimer,		&THIS::stateAppear);
		static StateFnPtr Effective	(GOSM_ID(Effective),root, &s.PlantAttackEffectiveTimer,		&THIS::stateEffective);
		static StateFnPtr Grow		(GOSM_ID(Grow),		root, &s.PlantAttackGrowTimer,			&THIS::stateGrow);
		static StateFnPtr Bite		(GOSM_ID(Bite),		root, &s.PlantAttackBiteTimer,			&THIS::stateBite);
		static StateFnPtr BiteCooldown(GOSM_ID(BiteCooldown),root, &s.PlantAttackBiteCooldownTimer,&THIS::stateBiteCooldown);
		static StateFnPtr Disappear	(GOSM_ID(Disappear),root, &s.PlantAttackDisappearTimer,		&THIS::stateDisappear);
		static State      Dead		(GOSM_ID(Dead),		root);

		Init 
		  >> Inactive;
		     Inactive 
		       >> Appear;
				  Appear [gc(Timeout)] 
		            >> Effective;
		               Effective [gc(BarrierContact)]  >> Grow;
		               Effective [!gc(BarrierContact)] << Grow;				
		               Effective [gc(BitableInRange)] /tf(BiteAction) 
		                 >> Bite;
		                    Bite [gc(Timeout)] >> BiteCooldown;
		               Effective [gc(Timeout)] << BiteCooldown;
					   Effective [gc(Starved) || gc(Beheaded)] >> Disappear;
		Dead                                                   << Disappear;
		
		StateMachineDef<THIS>::end();
		
		return smDef;
	}
	
	PlantAttack() {
		sm = new StateMachine<THIS>(getStateMachineDef());
	}
	
	virtual ~PlantAttack() {
		delete sm;
	}
	
	void update(float dTime) {
		sm->update(this, dTime);
	}
		
	/*
	 * STATE FNs
	 */
	void stateInactive()	{}
	void stateAppear()		{}
	void stateEffective()	{}
	void stateGrow()		{}
	void stateBite()		{}
	void stateBiteCooldown() {}
	void stateDisappear()	{}
	
	/*
	 * GUARD CONDITIONS
	 */
	bool gcBarrierContact(State* ctx)	{return gcBarrierContactFlag;}
	bool gcBitableInRange(State* ctx)	{return gcBitableInRangeFlag;}
	bool gcStarved(State* ctx)			{return gcStarvedFlag;}
	bool gcBeheaded(State* ctx)			{return gcBeheadedFlag;}
	bool gcTimeout(State* ctx)			{return sm->gcTimeout(ctx);}
	
	/*
	 * TRANSITION ACTIONS
	 */
	void tfBiteAction() {printf("\tBiteAction\n");}
	
public: /* debug flags */
	bool gcBarrierContactFlag;
	bool gcBitableInRangeFlag;
	bool gcStarvedFlag;
	bool gcBeheadedFlag;
	
protected:
	StateMachine<THIS>* sm;					// defined for each instance
	
protected:
	static StateMachineDef<THIS>* smDef;	// defined only once for all instances
};


#endif